#include <p32xxxx.h>
#include <plib.h>
#include "App1632_LCM.h"

#define LCM_INSTRUCTION	0x0000
#define LCM_DATA		0x0001

void LCM_Delay( unsigned int Loop )
{
 	unsigned int i = 0 , j = 0;

	for( j = 0 ; j < Loop ; j ++ )
		for( i = 0 ; i < 200 ; i ++ ); 
}

unsigned char LCM_IsBusy( void )
{
	unsigned char Temporary;

	PMPSetAddress( LCM_INSTRUCTION );
	Temporary = mPMPMasterReadByte( );		// Dummy Read for trigger PMP active.

	while( !INTGetFlag( INT_PMP ) );		// Wait PMP Complete.
	INTClearFlag( INT_PMP );

	Temporary = mPMPMasterReadByte( );

	return ( ( Temporary >> 7 ) & 0x01 ); 
}

void IntLCM_WriteData( unsigned char Data )
{
	while( LCM_IsBusy( ) );
	PMPSetAddress( LCM_DATA );
	PMPMasterWrite( Data );
}

void IntLCM_WriteInstruction( unsigned char Instruction )
{
	while( LCM_IsBusy( ) );
	PMPSetAddress( LCM_INSTRUCTION );
	PMPMasterWrite( Instruction );
}

void LCM_Init( void )
{
	mPMPOpen( PMP_ON | PMP_IDLE_CON | PMP_MUX_OFF | PMP_TTL | PMP_READ_WRITE_EN |
			  PMP_CS2_CS1_OFF | PMP_LATCH_POL_LO | PMP_CS2_POL_LO | PMP_CS1_POL_LO | PMP_WRITE_POL_HI | PMP_READ_POL_HI ,
			  PMP_IRQ_READ_WRITE | PMP_AUTO_ADDR_OFF | PMP_DATA_BUS_8 | PMP_MODE_MASTER1 |
			  PMP_WAIT_BEG_4 | PMP_WAIT_MID_15 | PMP_WAIT_END_4 ,
			  PMP_PEN_0 , 
			  PMP_INT_OFF | PMP_INT_PRI_0 );

	IntLCM_WriteInstruction( 0x30 );
	LCM_Delay( 25 );

	IntLCM_WriteInstruction( 0x30 );
	LCM_Delay( 1 );

	IntLCM_WriteInstruction( 0x30 );
	LCM_Delay( 1 );

	IntLCM_WriteInstruction( Disp_2Line_8Bit_5x8Dots );
	LCM_Delay( 1 );

	IntLCM_WriteInstruction( Disp_Off );
	LCM_Delay( 1 );

	IntLCM_WriteInstruction( Disp_Clear );
	LCM_Delay( 10 );

	IntLCM_WriteInstruction( Disp_Entry_Inc );
	LCM_Delay( 1 );

	IntLCM_WriteInstruction( Disp_On );
	LCM_Delay( 1 );
}

void LCM_SetCursor( unsigned char X, unsigned char Y )
{
	IntLCM_WriteInstruction( ( 0x80 + ( Y * 0x40 ) + X ) );
}

void LCM_PutASCII(unsigned char Ascii )
{
	IntLCM_WriteData( Ascii );
}

void LCM_PutHex(unsigned char Hex )
{
	unsigned char Temporary;

	Temporary = ( Hex >> 4 ) & 0x0f ;

	if ( Temporary > 9 ) 
		Temporary += 0x37;
    else
		Temporary += '0';

	IntLCM_WriteData( Temporary );

	Temporary = Hex & 0x0f;
	if ( Temporary > 9 )
		Temporary += 0x37;
    else Temporary += '0';
		IntLCM_WriteData( Temporary );
}

void LCM_PutROMString( const unsigned char *String )
{
	while ( *String != 0x00 )
		IntLCM_WriteData( *String++ );
}

void LCM_PutRAMString( unsigned char *String )
{
	while ( *String != 0x00 )
		IntLCM_WriteData( *String++ );
}

static unsigned char Disable_Zero = 1;

unsigned char LCM_BCD_Regulate( unsigned char BCD )
{
		if ( BCD == 0 ) 
		{
			if ( Disable_Zero == 1 ) 
				return ' ';
			else
				return '0';
		}

		else 
		{
			Disable_Zero = 0;
			return ( BCD += '0' );
		}		
}

void LCM_PutNumber( unsigned int Number , unsigned char Digit )
{
	unsigned char Temporary;
	Disable_Zero = 1;

	switch ( Digit )
	{
		default:
		case 5:
			Temporary = LCM_BCD_Regulate( ( Number % 100000 ) / 10000 );
			IntLCM_WriteData( Temporary );

		case 4:
			Temporary = LCM_BCD_Regulate( ( Number % 10000 ) / 1000 );
			IntLCM_WriteData( Temporary );

		case 3:
			Temporary = LCM_BCD_Regulate( ( Number % 1000 ) / 100 );
			IntLCM_WriteData( Temporary );

		case 2:
			Temporary = LCM_BCD_Regulate( ( Number % 100 ) / 10 );
			IntLCM_WriteData( Temporary );

		case 1:
			Disable_Zero = 0;
			Temporary = LCM_BCD_Regulate( Number % 10 );
			IntLCM_WriteData( Temporary );
	}
}
