#include <plib.h>
#include <p32xxxx.h>
#include ".\..\App1632_LCM.h"

/* Configuration Settings */
// OSC Input = HS w/PLL
// CPU Clock = ( ( OSC Input / 2 ) * 15 ) / 1
// PB Clock = CPU Clock / 1 
// Disabld WDT , ICSP/ICD = PGC2 , PGD2 
#pragma config FNOSC = PRIPLL , POSCMOD = HS
#pragma config FPLLIDIV = DIV_2 , FPLLMUL = MUL_15 , FPLLODIV = DIV_1
#pragma config FPBDIV = DIV_8
#pragma config FWDTEN = OFF , ICESEL = ICS_PGx2

#define SystemFrequency		60000000L
#define PBFrequency			SystemFrequency / 8
#define Timer1Tick			( PBFrequency / 256 / TogglesPerSec )
#define TogglesPerSec		1

const unsigned char LCMString1[ ] = "APP1632 WF002"; 
//const unsigned char LCMString2[ ] = "Rx:            "; 
const unsigned char LCMString2[ ] = "VR1:"; 
const unsigned char LCMString3[ ] = "VR2:"; 
unsigned int VR1Value = 0 , VR2Value = 0;

unsigned char UARTString1[ 32 ]; 
unsigned char Timer1TickFlag = 0;
unsigned char UART1_IsReceived = 0;
unsigned char UART2_IsReceived = 0;
unsigned char UART1_ReceiveBuffer = 0;
unsigned char UART2_ReceiveBuffer = 0;
unsigned char Hours = 0 , Mins = 0 , Secs = 0;
unsigned char enable_bootup_time = 0;
unsigned char enable_adc_read = 0;


#define UART_BUFFER_LEN	128
unsigned char UART1_RX_Buffer[UART_BUFFER_LEN];
unsigned char UART2_RX_Buffer[UART_BUFFER_LEN];
unsigned char *UART1_Read_ptr = UART1_RX_Buffer;
unsigned char *UART1_Write_ptr = UART1_RX_Buffer;
unsigned char *UART2_Read_ptr = UART2_RX_Buffer;
unsigned char *UART2_Write_ptr = UART2_RX_Buffer;



#define UARTRN171 UART1	/* UART1 for RN171 */
#define UARTChannel UART2	/* UART2 for APP1632 */ 


void UART1_PutROMString( unsigned char * String );
void UART2_PutROMString( unsigned char * String );

void UART1_WriteBuffer( unsigned char byte);
void UART2_WriteBuffer( unsigned char byte);
unsigned char UART1_ReadBuffer();
unsigned char UART2_ReadBuffer();
void ResetRN171( unsigned int reset_period);

void __ISR( _TIMER_1_VECTOR , ipl4 ) ISR_Timer1( void )
{
	Timer1TickFlag++;

	// MTimer_XСC
	mT1ClearIntFlag( );

	// LEDAϦVC	
	mPORTAToggleBits( BIT_0 | BIT_1 | BIT_2 | BIT_3 | BIT_4 | BIT_5 | BIT_6 | BIT_7 );

	// ϥC32 ADC FunctionҰADCഫC
	ConvertADC10( );
}

void __ISR( _UART_1_VECTOR , ipl5 ) ISR_UART1( void )
{
	// To Do : ϥC32 UART FunctionŪ쪺ơC
	//		   `N : Bƶ²ϥγ@ܼxsAYyqjɡA
	//		          zLLnUAHKƿ򥢡A
	//		          Ҧp:إߤ@Data QueueӦs񦬨쪺ơC
	UART1_ReceiveBuffer = UARTGetDataByte( UARTRN171 );
	UART1_WriteBuffer(UART1_ReceiveBuffer);
	UART1_IsReceived = 1;

	// MUART _XСC
	INTClearFlag( INT_U1RX );
}

void __ISR( _UART_2_VECTOR , ipl5 ) ISR_UART2( void )
{
	// To Do : ϥC32 UART FunctionŪ쪺ơC
	//		   `N : Bƶ²ϥγ@ܼxsAYyqjɡA
	//		          zLLnUAHKƿ򥢡A
	//		          Ҧp:إߤ@Data QueueӦs񦬨쪺ơC
	UART2_ReceiveBuffer = UARTGetDataByte( UARTChannel );

	UART2_IsReceived = 1;

	// MUART _XСC
	INTClearFlag( INT_U2RX );
}

int main( void )
{
	unsigned char Cursor = 0;

	// disable JTAG port
    DDPCONbits.JTAGEN = 0;

	// Pɯ = tήɯ / 8C
	OSCSetPBDIV( OSC_PB_DIV_8 );

	// ϥC32SYSTEM ConfigNCPUާ@̨ΤơAקFPBDIV]wC
	SYSTEMConfig( SystemFrequency , SYS_CFG_WAIT_STATES | SYS_CFG_PCACHE );
	// SYSTEMConfigPerformance( SystemFrequency  );
	// SYSTEMConfigWaitStates( SystemFrequency );

    // LEDI/O Port]wƦXAëwI/O PortAC
	mPORTASetPinsDigitalOut( BIT_0 | BIT_1 | BIT_2 | BIT_3 | BIT_4 | BIT_5 | BIT_6 | BIT_7 );
    mPORTASetBits( BIT_0 | BIT_1 | BIT_2 | BIT_3 );
	mPORTAClearBits( BIT_4 | BIT_5 | BIT_6 | BIT_7 );

	// ҥTimerAëw䤤_uvC
	ConfigIntTimer1( T1_INT_ON | T1_INT_PRIOR_4 );

	// Timer]wC
	OpenTimer1( T1_ON | T1_IDLE_CON | T1_GATE_OFF | T1_PS_1_256 | T1_SOURCE_INT , Timer1Tick );

	// LCMlơC
	LCM_Init( );

	// ]wCСAXrC
	LCM_SetCursor( 0 , 0 );
	LCM_PutROMString( LCMString1 );
	//LCM_SetCursor( 0 , 1 );
	//LCM_PutROMString( LCMString2 );

	// To Do : ϥC32 UART FunctionlUARTC
	// Ref. \C32_Doc\pic32-lib-help\UART-PLIB-Help.chm
		// void UARTConfigure( UART_MODULE id , UART_CONFIGURATION flags );
		// void UARTSetFifoMode( UART_MODULE id , UART_FIFO_MODE mode );
		// void UARTSetLineControl( UART_MODULE id , UART_LINE_CONTROL_MODE mode );
		// UINT32 UARTSetDataRate( UART_MODULE id ,UINT32 sourceClock ,UINT32 dataRate );
		// void UARTEnable( UART_MODULE id , UART_ENABLE_MODE mode );

  	// MPLAB C32 1.11BP1.11A UART enum Label 
		// 1.11B				// 1.11A
		// #define UART2		UART3A
		// #define UART3		UART2A
		// #define UART4		UART1B
		// #define UART5		UART3B
		// #define UART6		UART2B
		// UART1 P UART1A wbHeader Filewq, (Ex: Ref. p32mx795f512l.h )

	// Init UART1
	UARTConfigure( UARTRN171 , UART_ENABLE_PINS_TX_RX_ONLY );
	UARTSetFifoMode( UARTRN171 , UART_INTERRUPT_ON_TX_NOT_FULL | UART_INTERRUPT_ON_RX_NOT_EMPTY );
	UARTSetLineControl( UARTRN171 , UART_DATA_SIZE_8_BITS | UART_PARITY_NONE | UART_STOP_BITS_1 ); 
	UARTSetDataRate( UARTRN171 , PBFrequency , 9600 );
	UARTEnable( UARTRN171 , UART_ENABLE | UART_PERIPHERAL | UART_TX | UART_RX );

	// Init UART2
	UARTConfigure( UARTChannel , UART_ENABLE_PINS_TX_RX_ONLY );
	UARTSetFifoMode( UARTChannel , UART_INTERRUPT_ON_TX_NOT_FULL | UART_INTERRUPT_ON_RX_NOT_EMPTY );
	UARTSetLineControl( UARTChannel , UART_DATA_SIZE_8_BITS | UART_PARITY_NONE | UART_STOP_BITS_1 ); 
	UARTSetDataRate( UARTChannel , PBFrequency , 9600 );
	UARTEnable( UARTChannel , UART_ENABLE | UART_PERIPHERAL | UART_TX | UART_RX ); 

	// To Do : ϥC32 UART FunctionҥUARTAëw䤤_uvC
		// Ref. \C32_Doc\pic32-lib-help\INT-PLIB-Help.chm
		// void INTSetVectorPriority( INT_VECTOR vector , INT_PRIORITY priority );
		// void INTSetVectorSubPriority( INT_VECTOR vector , INT_SUB_PRIORITY subPriority );
		// void INTEnable( INT_SOURCE source , INT_EN_DIS enable );

  	// MPLAB C32 1.11BP1.11A UART INT Label 
		// 1.11B							// 1.11A
		// #define INT_UART_1_VECTOR		INT_UART_1A_VECTOR
		// #define INT_UART_2_VECTOR		INT_UART_3A_VECTOR
		// #define INT_UART_4_VECTOR		INT_UART_1B_VECTOR
		// #define INT_UART_3_VECTOR		INT_UART_2A_VECTOR
		// #define INT_UART_6_VECTOR		INT_UART_2B_VECTOR
		// #define INT_UART_5_VECTOR		INT_UART_3B_VECTOR

	// Init UART 1 Int
	INTSetVectorPriority( INT_UART_1_VECTOR , INT_PRIORITY_LEVEL_5 );
	INTSetVectorSubPriority( INT_UART_1_VECTOR , INT_SUB_PRIORITY_LEVEL_0 );
	INTEnable( INT_U1RX , INT_ENABLED );

	// Init UART 2 Int
	INTSetVectorPriority( INT_UART_2_VECTOR , INT_PRIORITY_LEVEL_5 );
	INTSetVectorSubPriority( INT_UART_2_VECTOR , INT_SUB_PRIORITY_LEVEL_0 );
	INTEnable( INT_U2RX , INT_ENABLED );

	// ϥC32 ADC10 Function]wADCC
	OpenADC10( ADC_MODULE_ON | ADC_IDLE_CONTINUE | ADC_FORMAT_INTG16 | ADC_CLK_MANUAL | ADC_AUTO_SAMPLING_ON | ADC_SAMP_OFF ,
			   ADC_VREF_AVDD_AVSS | ADC_OFFSET_CAL_DISABLE | ADC_SCAN_ON | ADC_SAMPLES_PER_INT_2 | ADC_ALT_BUF_OFF | ADC_ALT_INPUT_OFF ,
			   ADC_SAMPLE_TIME_31 | ADC_CONV_CLK_SYSTEM | ADC_CONV_CLK_32Tcy ,
			   ENABLE_AN2_ANA | ENABLE_AN4_ANA , 
			   SKIP_SCAN_AN0 | SKIP_SCAN_AN1 | SKIP_SCAN_AN3 | SKIP_SCAN_AN5 | SKIP_SCAN_AN6 | SKIP_SCAN_AN7 |
			   SKIP_SCAN_AN8 | SKIP_SCAN_AN9 | SKIP_SCAN_AN10 | SKIP_SCAN_AN11 | SKIP_SCAN_AN12 | SKIP_SCAN_AN13 |
			   SKIP_SCAN_AN14 | SKIP_SCAN_AN15 );

	// ϥC32 I/O Port FunctionNTADqD]wJC
	PORTSetPinsAnalogIn( IOPORT_B , BIT_2 | BIT_4 );
	
	// set LCD cursor position
	Cursor = 3;

    INTEnableSystemMultiVectoredInt( );

	// Boot up string to UART console
	UART2_PutROMString("\r\n***APP1632 WF002 with RN171-PICTAIL***\r\n");

	// reset RN171 by hardware
	UART2_PutROMString("Reset RN171...\r\n");
	ResetRN171(1024);
	/*mPORTFSetPinsDigitalOut( BIT_0  );
	mPORTFClearBits( BIT_0  );
	int i;
	for( i = 0 ; i < 1024 ; i++ );
	mPORTFSetBits( BIT_0  );
	mPORTFSetPinsDigitalIn( BIT_0  );*/

	while( 1 )
	{

		/*if( UART1_IsReceived )
		{
			UART1_IsReceived = 0;

			// ]wCСAXrC
			//LCM_SetCursor( Cursor++ , 1 );
			//LCM_PutASCII( UART1_ReceiveBuffer );

			//if( Cursor > 15 ) Cursor = 3;
			while( !UARTTransmitterIsReady( UARTChannel ) );
	    	UARTSendDataByte( UARTChannel , UART1_ReceiveBuffer );
			
		}*/

		if( UART1_Read_ptr != UART1_Write_ptr )
		{
			//UART1_IsReceived = 0;

			// ]wCСAXrC
			//LCM_SetCursor( Cursor++ , 1 );
			//LCM_PutASCII( UART1_ReceiveBuffer );

			//if( Cursor > 15 ) Cursor = 3;
			unsigned char data_byte;
			
			// read UART1 RX buffer
			data_byte = UART1_ReadBuffer();
			while( !UARTTransmitterIsReady( UARTChannel ) );
	    	UARTSendDataByte( UARTChannel , data_byte );
			
		}

		// UART ?
		if( UART2_IsReceived )
		{
			UART2_IsReceived = 0;

			// ]wCСAXrC
			//LCM_SetCursor( Cursor++ , 1 );
			//LCM_PutASCII( UART2_ReceiveBuffer );

			//if( Cursor > 15 ) Cursor = 3;
			switch(UART2_ReceiveBuffer)
			{
				case 'a':
					UART1_PutROMString("apmode\r");
					break;
				case 'c':
					UART1_PutROMString("$$$");
					break;
				case 'j':
					UART1_PutROMString("join\r");
					break;
				case 'r':
					UART1_PutROMString("reboot\r");
					break;
				case 't':
					enable_bootup_time = !enable_bootup_time;
					break;
				case 'v':
					enable_adc_read = !enable_adc_read;
					break;
				default:
					;
			}

		}

		// ADC ഫ ?
		if( INTGetFlag( INT_AD1 ) )
		{
			// To Do : ϥC32 ADC10 FunctionŪADCഫƭȡC
			VR1Value = ( unsigned int ) ReadADC10( 0 );
			VR2Value = ( unsigned int ) ReadADC10( 1 );

			// MADC_X
			INTClearFlag( INT_AD1 );

			// ]wCСAXADCƭȡC
			LCM_SetCursor( 0 , 1 );
			LCM_PutROMString( LCMString2 );
			LCM_PutNumber( VR1Value , 4 );
			LCM_PutASCII( ',' );
			LCM_PutROMString( LCMString3 );
			LCM_PutNumber( VR2Value >> 1 , 3 );
			
		}

		// BzC
		if( Timer1TickFlag >= 1 ) 	
		{
			if( ++Secs > 59 ) 
			{
				Secs = 0;
				if( ++Mins > 59 )
				{
					Mins = 0;
					if( ++Hours > 23 )
						Hours = 0;
				}
			}
			
			if(enable_bootup_time)
			{
				sprintf( UARTString1 , "Now Time %2d:%2d:%2d\r\n" ,
					 Hours , Mins , Secs );

 				// To Do : ϥC32 I/O UART FunctionNrzLUARTXC
				UART1_PutROMString( UARTString1 );
			}
			if(enable_adc_read)
			{
				sprintf( UARTString1 , "ADC reading VR1:%2d  VR2:%2d\r\n" ,
					 VR1Value  , VR2Value  );

 				// To Do : ϥC32 I/O UART FunctionNrzLUARTXC
				UART1_PutROMString( UARTString1 );				
			}

			Timer1TickFlag--;
		}
	}

	while( 1 );
}

void UART2_PutROMString( unsigned char * String ) 
{
	while ( *String != 0x00 )
	{
		while( !UARTTransmitterIsReady( UARTChannel ) );
	    UARTSendDataByte( UARTChannel , *String++ );
	}
}

void UART1_PutROMString( unsigned char * String ) 
{
	while ( *String != 0x00 )
	{
		while( !UARTTransmitterIsReady( UARTRN171 ) );
	    UARTSendDataByte( UARTRN171 , *String++ );
	}
}

void UART1_WriteBuffer( unsigned char byte)
{
	*UART1_Write_ptr++ = byte;
	if (UART1_Write_ptr == UART1_RX_Buffer+UART_BUFFER_LEN)
		UART1_Write_ptr = UART1_RX_Buffer;
}
void UART2_WriteBuffer( unsigned char byte)
{
	*UART2_Write_ptr++ = byte;
	if (UART2_Write_ptr == UART2_RX_Buffer+UART_BUFFER_LEN)
		UART2_Write_ptr = UART2_RX_Buffer;
}
unsigned char UART1_ReadBuffer()
{
	unsigned char byte;
	byte = *UART1_Read_ptr++;
	if (UART1_Read_ptr == UART1_RX_Buffer+UART_BUFFER_LEN)
		UART1_Read_ptr = UART1_RX_Buffer;
	return byte;
}
unsigned char UART2_ReadBuffer()
{
	unsigned char byte;
	byte = *UART2_Read_ptr++;
	if (UART2_Read_ptr == UART2_RX_Buffer+UART_BUFFER_LEN)
		UART2_Read_ptr = UART2_RX_Buffer;
	return byte;
}

void ResetRN171( unsigned int reset_period)
{
	mPORTFSetPinsDigitalOut( BIT_0  );
	mPORTFClearBits( BIT_0  );
	int i;
	for( i = 0 ; i < reset_period ; i++ );
	mPORTFSetBits( BIT_0  );
	mPORTFSetPinsDigitalIn( BIT_0  );
}